/*** ezt_funcs.c:begin ***/

#ifdef EZT_ERRORPRINTF
static int
Ezt_ErrorPrintf (Ezt_Interp *interp, const char *format, ...) {
	va_list args;
	char buf[200];
	if (interp == NULL) { return EZT_ERROR; }
	va_start(args, format);
	vsnprintf(buf, sizeof buf, format, args);
	Ezt_SetResult(Ezt_NewStr(buf));
	va_end(args);
	return EZT_ERROR;
}
#endif

#ifdef EZT_REPORTERROR
static int
Ezt_ReportError (Ezt_Interp *interp, ...) {
	va_list args;
	const char *arg;
	Ezt_Obj *result;
	
	if (interp == NULL) { return EZT_ERROR; }
	result = Ezt_NewStr("");
	Ezt_IncrRefCount(result);
	va_start(args, interp);
	while ((arg = va_arg(args, char *)) != NULL) {
		Ezt_Append(result, arg, -1);
	}
	va_end(args);
	Ezt_SetResult(result);
	Ezt_DecrRefCount(result);
	return EZT_ERROR;
}
#endif

#ifdef EZT_EZTWRONGNUMARGS
static int
Ezt_EztWrongNumArgs (Ezt_Interp *interp, int objc, Ezt_Obj *const objv[], const char *message) {
	Ezt_TJWrongNumArgs(interp, objc, objv, message);
	return EZT_ERROR;
}
#endif

#ifdef EZT_TIMEVAL2DBL
static Ezt_Obj *
Ezt_TimevalToDoubleObj (struct timeval *tv) {
	return Ezt_NewDouble(tv->tv_sec + tv->tv_usec / 1000000.0);
}
#endif

#ifdef EZT_STR2TIMEVAL
static int
Ezt_StrToTimeval (Ezt_Interp *interp, const char *str, struct timeval *tv) {
	Ezt_Obj *o;
	int res;
	o = Ezt_NewStr(str);
	Ezt_IncrRefCount(o);
	res = Ezt_DoubleObjToTimeval(interp, o, tv);
	Ezt_DecrRefCount(o);
	return res;
}
#endif

#ifdef EZT_DBL2TIMEVAL
static int
Ezt_DoubleObjToTimeval (Ezt_Interp *interp, Ezt_Obj *obj, struct timeval *tv) {
	double d;

	if (Ezt_GetDouble(interp, obj, &d) != EZT_OK) { return EZT_ERROR; }

	tv->tv_sec = (time_t) d;	/* truncate to get secs */
	d -= tv->tv_sec;		/* remove secs */
	d *= 1000000;			/* scale up */
	tv->tv_usec = (suseconds_t) d;	/* cast/truncate */

	return EZT_OK;
}
#endif


#ifdef FOR_JIM
# if 0
static Jim_Obj *
Ezt_NewIntObj (Jim_Interp *interp, int i) {
	return Jim_NewWideObj(interp, (ezt_wide) i);
}
# endif

# ifdef EZT_JIM_GETINT
static int
Ezt_GetIntFromJimObj (Jim_Interp *interp, Jim_Obj *objPtr, int *intPtr) {
	jim_wide w;
	if (Jim_GetWide(interp, objPtr, (intPtr != NULL ? &w : NULL)) != JIM_OK) { return JIM_ERR; }
	if (intPtr != NULL) {
		*intPtr = (int) w;
	}
	return JIM_OK;
}
# endif
#endif /* FOR_JIM */


#ifdef EZT_ASSOCDATA_KEY

static void
Ezt_GoodbyeCruelInterp (ClientData clientData, Tcl_Interp *interp) {
	Tcl_Obj *o;
	EztJunk *j;

	o = (Tcl_Obj *) clientData;
	j = (EztJunk *) Tcl_GetByteArrayFromObj(o, NULL);

#ifdef EZT_BYE
	EZT_BYE(j->pile);
#endif

	Tcl_DecrRefCount(j->pile);
	Tcl_DecrRefCount(o);

	/* Prevent being called recursively. Happens when called from Unload. */
	Tcl_SetAssocData(interp, EZT_ASSOCDATA_KEY, NULL, (ClientData) o);
	Tcl_DeleteAssocData(interp, EZT_ASSOCDATA_KEY);
}

static EztJunk *
Ezt_GetJunk (Tcl_Interp *interp) {
	Tcl_Obj *o;
	if ((o = (Tcl_Obj *) Tcl_GetAssocData(interp, EZT_ASSOCDATA_KEY, NULL)) == NULL) {
		EztJunk j;
		j.counter = 0;
		j.pile = Tcl_NewDictObj();
		Tcl_IncrRefCount(j.pile);
		o = Tcl_NewByteArrayObj((const unsigned char *) &j, sizeof(j));
		Tcl_IncrRefCount(o);
		Tcl_SetAssocData(interp, EZT_ASSOCDATA_KEY, Ezt_GoodbyeCruelInterp, (ClientData) o);
	}
	return (EztJunk *) Tcl_GetByteArrayFromObj(o, NULL);
}

static int
Ezt_PutIt (Tcl_Interp *interp, Tcl_Obj *what, Tcl_Obj *val) {
	EztJunk *j;
	j = Ezt_GetJunk(interp);
	Tcl_DictObjPut(NULL, j->pile, what, val);
	return 1;
}

static void
Ezt_RemoveIt (Tcl_Interp *interp, Tcl_Obj *what) {
	EztJunk *j;
	j = Ezt_GetJunk(interp);
	Tcl_DictObjRemove(NULL, j->pile, what);
}

static Tcl_Obj *
Ezt_GetIt (Tcl_Interp *interp, Tcl_Obj *what) {
	EztJunk *j;
	Tcl_Obj *o;
	j = Ezt_GetJunk(interp);
	Tcl_DictObjGet(NULL, j->pile, what, &o);
	return o;
}

static unsigned long
Ezt_NextCounter (Tcl_Interp *interp) {
	EztJunk *j;
	j = Ezt_GetJunk(interp);
	return j->counter++;
}

# ifdef EZT_FUNC_JUNKPILEASLIST
/*
 * The returned object has a refcount of 1.
 */
static Tcl_Obj *
Ezt_JunkPileAsList (Tcl_Interp *interp) {
	EztJunk *j;
	Tcl_Obj *o;
	Tcl_DictSearch s;
	Tcl_Obj *k = NULL;
	int d = 0;

	j = Ezt_GetJunk(interp);
	o = Tcl_NewListObj(0, NULL);
	Tcl_IncrRefCount(o);
	Tcl_DictObjFirst(NULL, j->pile, &s, &k, NULL, &d);
	while (!d) {
		Tcl_ListObjAppendElement(NULL, o, k);
		Tcl_DictObjNext(&s, &k, NULL, &d);
        }
	Tcl_DictObjDone(&s);
	return o;
}
# endif /* EZT_FUNC_JUNKPILEASLIST */

#endif /* EZT_ASSOCDATA_KEY */

/*** ezt_funcs.c:end ***/
